#include <Array.au3>

Opt("MustDeclareVars", 1)

Demo_1()


Func Demo_1()
  Local $sFilePath

  $sFilePath = @ScriptDir & "\_test file.au3"

  Local $hTimer = TimerInit()

  Local $iRounds = 10
  For $i = 1 To $iRounds
    Local $aFuncList = ReadFunctionHeads($sFilePath)
  Next

  If @error Then
    MsgBox(0, "Error", @error & " = file not opened in read mode or other error")
  Else
    ConsoleWrite(@CRLF & '! Average execution time: ' _
      & TimerDiff($hTimer) / $iRounds & ' ms. Rounds: ' & $iRounds & @CRLF & @CRLF)

    Local $sDemoHint = _
      "        ---- Die Spitzklammern dienen nur zur Verdeutlichung," _
      & " ob Leerzeichen vor oder hinter der Syntax-Zeile stehen. ----        "
      ; The angle brackets only serve to clarify whether spaces are before or after the syntax line.

    _ArrayDisplay($aFuncList, "File:  " & $sFilePath, "", 0, Default, $sDemoHint)
  EndIf
EndFunc


; ==============================================================================
; ReadFunctionHeads
;
; Author:         Professor Bernd.
; Date:           2020-07-06.
; Syntax:         ReadFunctionHeads(Const $_sFilePath)
; Parameters:     $_sFilePath - the file path as a string
; Return values:  Success - an array with function names and parameter lists
;                 Failure - @error
; Description:
;
;   Routine zum Auslesen von Funktions-Namen und deren Parameter-Liste aus den
;   Funktions-Deklarationen einer Datei.
;
;   Ausgelesen werden alle gltigen Funktions-Kpfe einer Datei. Daraus werden
;   die Funktions-Namen mit zugehriger Parameter-Liste gefiltert und in einer
;   Schreibweise dargestellt, die der aus der AutoIt Hilfe hnelt.
;
;   - - - -
;
;   Routine to read function names and their parameter list from the function
;   declarations of a file.
;
;   All valid function headers of a file are read out. The function names
;   with the associated parameter list are filtered and displayed in a
;   notation similar to the AutoIt help.
; ------------------------------------------------------------------------------
Func ReadFunctionHeads(Const $_sFilePath)

  ; Datei komplett einlesen.
  ; Datei auf jeden Fall mit @CRLF abschlieen, damit die folgenden Erkennungen
  ;  auch in der letzten Zeile funktionieren.
  ; In any case, close the file with @CRLF so that the following recognitions
  ;  also work in the last line.
  Local $sAllLines = FileRead($_sFilePath) & @CRLF

  If $sAllLines = @CRLF Then Return SetError(1, 0, '')

  ; Entferne #Region- und #EndRegion-Direktiven.
  ; Remove #Region and #EndRegion directives.
  ; Author: DXRW4E, 2013-02-11.
  ; https://www.autoitscript.com/forum/topic/148213-parse-all-includes-in-a-script-file/?do=findComment&comment=1052998
  ; Modified by Professor Bernd, 2020-07-06.
  $sAllLines = StringRegExpReplace($sAllLines, '(?i)\R\h*(?:#Region|#EndRegion)(?!\w)[^\R]*\R', "")

  ; Entferne einzeilige Kommentare.
  ; Strip single line comments.
  ; Author: mikell, 2013-12-13.
  ; https://www.autoitscript.com/forum/topic/157255-regular-expression-challenge-for-stripping-single-comments/?do=findComment&comment=1138772
  $sAllLines = StringRegExpReplace($sAllLines, '(?m)^((?:[^''";]*([''"]).*?\2)*[^;]*);?.*$', '$1')

  ; Entferne einfache und verschachtelte Block-Kommentare.
  ; Remove simple and nested block comments (#cs or #comment-start & #ce or #comment-end).
  ; Author: AspirinJunkie, 2020-06-29.
  ; https://autoit.de/thread/87036-verschachtelte-block-kommentare-entfernen-mit-regexp/?postID=700789#post700789
  $sAllLines = StringRegExpReplace($sAllLines, '(?xmi) (^\h*\# (?>cs|comments-start)\b (?sU:(?>(?R)|.)*) ^\h*\#(?>ce|comments-end)\b.*\R )', '')

  ; Entferne Zeilenfortsetzungszeichen " _", nachfolgenden Leerraum und den Zeilenumbruch.
  ; Remove line continuation characters " _", following whitespace and the newline sequence.
  $sAllLines = StringRegExpReplace($sAllLines, '\h+_\h*\R+?', ' ')

;     ; Ersetzen aller "' & '" Kombinationen, die durch das Entfernen von " _"
;     ;  entstanden sind, wenn Strings ber 2 Zeilen fortgesetzt wurden.
;     ;  Leider scheint das etwas Zeit zu kosten, fr wenig Nutzen. 2020-07-03.
;     ; Replacing all "' & '" combinations that were created by removing " _"
;     ;  when strings were continued over 2 lines. Unfortunately this seems
;     ;  to take some time, for small benefit. 2020-07-03.
;     $sAllLines = StringRegExpReplace($sAllLines, '"\h*&\h*''(.*)''', '\1"')
;     $sAllLines = StringRegExpReplace($sAllLines, "'\h*&\h*""(.*)""", "\1'")

  ; Entferne alle Zeilen, die nicht mit "Func " oder "EndFunc" beginnen.
  ; Remove all lines that do not begin with "Func " or "EndFunc".
  ; Based on: "Delete all lines that do not have the contents of $sSearch4 present."
  ; Author: Malkey, 2017-11-30.
  ; https://www.autoitscript.com/forum/topic/191371-autoit-delete-all-lines-from-a-txt-file-except-some/?do=findComment&comment=1372776
  ; Modified by Professor Bernd, 2020-06-28, 2020-07-03.
  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?mi)^\h*Func\h+.*\R?(*SKIP)(*F)|^\h*EndFunc.*\R?(*SKIP)(*F)|^.*\R?', "")

  ; Leerzeichen vor und hinter folgenden Zeichen einfgen (aber nur wenn sie nicht in
  ; Anfhrungzeichen stehen): Runde Klammern, Gleich-Zeichen, AutoIt Operatoren.
  ; Insert spaces before and after the following characters (but only if they are not in
  ; quotation marks): Parentheses, equal signs, AutoIt operators.
  ; Author: AspirinJunkie, 2020-07-01.
  ; https://autoit.de/thread/87036-regexp-verschachtelte-block-kommentare-entfernen-char-nicht-in-string-ersetzen/?postID=700817#post700817
; 	$sAllLines = StringRegExpReplace($sAllLines, _
;     '(?x)( (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \h* (?: <> | [&\+\-\*\/=><]? = | [\(\)\*+\-\/&^<>?:] | \b(?i:And|Or|Not)\b) \h*)', ' \1 ')
  ; Without "And, Or, Not".
  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?x)( (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \h* (?: <> | [&\+\-\*\/=><]? = | [\(\)\*+\-\/&^<>?:] ) \h*)', ' \1 ')

  ; Formatiere "And", "Or" und "Not" (aber nur wenn sie nicht in Anfhrungzeichen stehen).
  ; Format "And", "Or" and "Not" (but only if they are not in quotation marks).
  ; Author: AspirinJunkie, 2020-07-01.
  ; https://autoit.de/thread/87036-regexp-verschachtelte-block-kommentare-entfernen-char-nicht-in-string-ersetzen/?postID=700826#post700826
  ; Modified by Professor Bernd, 2020-07-02.
  $sAllLines = StringTrimRight(StringRegExpReplace($sAllLines & ':and=And:or=Or:not=Not', _
    '(?sxi)(?: (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \b(and|or|not)\b ) (?=.*:\1=(\w+)\b)', '\2'), 22)
;   ; Gleiche Anweisung, andere Schreibweise (Single und Double-Quotes in 2 Strings getrennt, statt jedes Zeichen zu escapen.
;   ; Same instruction, different spelling (single and double quotes separated into 2 strings instead of escaping each character.
;   $sAllLines = StringTrimRight(StringRegExpReplace($sAllLines & ':and=And:or=Or:not=Not', _
;     '(?sxi)(?: (?>"(?> [^"]+ | "" )*" |' & " '(?> [^']+ | '' )*') (*SKIP)(*FAIL) | \b(and|or|not)\b ) (?=.*:\1=(\w+)\b)", '\2'), 22)

  ; Formatiere "Const" und "ByRef" (aber nur wenn sie nicht in Anfhrungzeichen stehen).
  ; Format "Const" and "ByRef" (but only if they are not in quotation marks).
  ; Modified with pattern from AspirinJunkie by Professor Bernd, 2020-06-30.
  ; Nur Treffer auerhalb von Quotes ersetzen.
	$sAllLines = StringRegExpReplace($sAllLines, _
    '(?xi)( (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \bconst\b )', ' Const ')
	$sAllLines = StringRegExpReplace($sAllLines, _
    '(?xi)( (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \bbyref\b )', ' ByRef ')

  ; Formatiere Kommas folgendermaen (aber nur wenn sie nicht in Anfhrungzeichen stehen):
  ;  Vor dem Komma alle Leerraum entfernen, hinter dem Komma ein Leerzeichen einfgen.
  ; Format commas as follows (but only if they are not in quotation marks):
  ;  Remove all whitespace before the comma, insert a space after the comma.
  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?x)( (?>"(?> [^"]+ | "" )*" | ''(?> [^'']+ | '''' )*'') (*SKIP)(*FAIL) | \h*, )', ', ')

  ; Mehrfachen Leerraum durch ein einzelnes Leerzeichen ersetzen.
  ; (Dabei werden auch Tabs durch Leerzeichen ersetzt.)
  ; Replace multiple whitespace with a single space.
  $sAllLines = StringRegExpReplace($sAllLines, "(\h+)", ' ')

  ; Zeilen mit gltigem Funktions-Kopf in ein Array speichern, wenn die nchste
  ; Zeile mit "EndFunc" beginnt. Dabei String "Func " entfernen.
  ; Hinweis; Das funktioniert NUR, weil es an dieser Stelle nur noch Zeilen gibt,
  ;          die mit "Func " oder "EndFunc" beginnen! 2020-06-28, 2020-07-03.
  ; Store lines with a valid function header in an array if the next line
  ; starts with "EndFunc". Thereby remove string "Func ".
  ; Hint: This works ONLY because at this point there are only lines beginning
  ;       with "Func " or "EndFunc"! 2020-06-28, 2020-07-03.
  ; Based on code from Bitnugger.
  ; Modified and "EndFunc" part added by Professor Bernd, 2020-07-03. - (?i) bedeuted case insensitive (Tipp von alpines).
;   Local $aFuncs = StringRegExp($sAllLines, _
;     '(?i)Func (\w+\h*\(.*\)\h*\R)\h*EndFunc.*\R', $STR_REGEXPARRAYGLOBALMATCH) ; Zeile inkl. @CRLF
  Local $aFuncs = StringRegExp($sAllLines, _
    '(?i)Func (\w+\h*\(.*\)\h*)\R\h*EndFunc.*\R', $STR_REGEXPARRAYGLOBALMATCH) ; Zeile ohne @CRLF

      ; Alle einfachen Anfhrungszeichen durch doppelte Anfhrungszeichen ersetzen.
      ; => NICHT gut bei z. B. '"' !
      ; Replace all single quotation marks with double quotation marks.
      ; => NOT good with e.g. '"' !
      ;$sAllLines = StringReplace($sAllLines, "'", '"', 0, $STR_NOCASESENSEBASIC)

  ; Array mit den gefundenen Funktions-Kpfen durchlaufen.
  ;  Go through the array with the found function heads.
  ; Die Spitzklammern dienen nur der Verdeutlichung, ob Leerzeichen vor oder hinter der Syntax-Zeile stehen.
  ;  The angle brackets only serve to clarify whether spaces are before or after the syntax line.
  For $i = 0 To UBound($aFuncs) - 1 Step 1
    $aFuncs[$i] = '>' & $aFuncs[$i] & '<'
  Next

  Return SetError(@error, @extended, @error ? @error : $aFuncs)
EndFunc ; ==> ReadFunctionHeads

